
#include "vssynthbase.h"

//This is the base class for a virtual sampler synthesizer plugin
//For a complete synth you should override all virtual functions.
//At a minimum, you must override the "NoteOn", "NoteOff" and
//"GetBuffer" functions.
class VSDemoSynth: public VSSynthBase
{
public:	 
	VSDemoSynth(int ASampleRate);   
	virtual ~VSDemoSynth();

	virtual void SetProgram(int NewValue);
	virtual void GetProgramName(int Index,char *Result);
	virtual int GetProgramCount();
	virtual int GetParamCount(int GroupIndex);
	virtual void GetParamLabel(int GroupIndex,int Index,char *Result);
	virtual void GetParamDisplay(int GroupIndex,int Index,char *Result);
	virtual void GetParamValue(int GroupIndex,int Index,float *Result);
	virtual void SetParamValue(int GroupIndex,int Index,float NewValue);
	virtual int GetGroupCount();
	virtual void GetGroupName(int Index,char *Result);
	virtual void NoteOn(int Note,int Velocity,int ScaleCents);
	virtual void NoteOff();
	virtual void LoadParams(void *Params,int ParamsLen);
	virtual void GetParamsInfo(int *ParamsLen);
	virtual void StoreParams(void *Params,int ParamsLen);
	virtual int GetBuffer(float *Buffer,int SampleCount);
	virtual void InitializeProgram();
	virtual void GetFileExt(char *Result);
	virtual int SaveProgram(char *FileName);
	virtual void GetProgramInfo(char *FileName,int *ProgramCount,int *ParamsLen);
	virtual int LoadProgram(char *FileName,PSynthProgramInfo Result);
private:
	float Osc1WaveForm;
	float Osc1Detune;
	float Osc1Volume;
	float Osc2WaveForm;
	float Osc2Detune;
	float Osc2Volume;
	float MasterGain;

	//MIDI buffer
	int MidiNote;
	double FreqOsc1;
	double FreqOsc2;
	double PhaseOsc1;
	double PhaseOsc2;
	double Volume;
	int MidiVelocity;
	int MidiScaleCents;
	bool MidiNoteIsOn;
};


