//*******************************************************************
//*																	*
//* Virtual Sample Demo Synthesizer PlugIn							*
//*																	*
//*																	*	
//*																	*	
//*	This sample code emulates a very simple synthesizer with 2   	*	
//*	oscilators and 2 waveforms for each oscillator.  				*	
//* It should give you an idea how to write a Virtual Sampler       *
//* Synthesizer plugin.  											*	
//*																	*	
//*******************************************************************

Library synthdemo;

Uses
  Windows,
  synthbase in 'synthbase.pas';

{$R *.RES}

Type
   TVSDemoSynth=Class(TVSSynthBase)
         Private
           Osc1WaveForm:Float;
           Osc1Detune:Float;
           Osc1Volume:Float;
           Osc2WaveForm:Float;
           Osc2Detune:Float;
           Osc2Volume:Float;
           MasterGain:Float;

           //MIDI buffer
           MidiNote:LongInt;
           FreqOsc1:Double;
           FreqOsc2:Double;
           PhaseOsc1:Double;
           PhaseOsc2:Double;
           Volume:Double;
           MidiVelocity:LongInt;
           MidiScaleCents:LongInt;
           MidiNoteIsOn:Boolean;
         Public
           Constructor Create(ASampleRate:LongInt);Override;
           Destructor Destroy;Override;

           Procedure SetProgram(NewValue:LongInt);Override;
           Procedure GetProgramName(Index:LongInt;Var Result:String);Override;
           Function GetProgramCount:LongInt;Override;
           Function GetParamCount(GroupIndex:LongInt):LongInt;Override;
           Procedure GetParamLabel(GroupIndex,Index:LongInt;Var Result:String);Override;
           Procedure GetParamDisplay(GroupIndex,Index:LongInt;Var Result:String);Override;
           Procedure GetParamValue(GroupIndex,Index:LongInt;Var Result:Float);Override;
           Procedure SetParamValue(GroupIndex,Index:LongInt;NewValue:Float);Override;
           Function GetGroupCount:LongInt;Override;
           Procedure GetGroupName(Index:LongInt;Var Result:String);Override;
           Procedure NoteOn(Note,Velocity,ScaleCents:LongInt);Override;
           Procedure NoteOff;Override;
           Procedure LoadParams(Params:Pointer;ParamsLen:LongInt);Override;
           Procedure GetParamsInfo(Var ParamsLen:LongInt);Override;
           Procedure StoreParams(Params:Pointer;ParamsLen:LongInt);Override;
           Function GetBuffer(Buffer:PFloatArray;SampleCount:LongInt):LongInt;Override;
           Procedure InitializeProgram;Override;
           Procedure GetFileExt(Var Result:String);Override;
           Function SaveProgram(Var FileName:String):LongInt;Override;
           Procedure GetProgramInfo(Const FileName:String;Var ProgramCount,ParamsLen:LongInt);Override;
           Function LoadProgram(Var FileName:String;Var Res:TSynthLoadProgramRec):LongInt;Override;
   End;

//*******************************************************************
//*																	*
//* Member Functions												*
//*																	*
//*																	*	
//*******************************************************************

//Frequency table in Hz for 0..128 MIDI notes
Const FreqTableInitialized:Boolean=false;
Const FreqTableSize=128; //128 MIDI notes
Var FreqTable:Array[0..FreqTableSize-1] Of Float;

//WaveForms tables
Const WaveFormsInitialized:Boolean=false;
Const WaveTableSize=2048;
Var PulseTable:Array[0..WaveTableSize-1] Of Float;
Var SawToothTable:Array[0..WaveTableSize-1] Of Float;

//Constructor - setup
Constructor TVSDemoSynth.Create(ASampleRate:LongInt);
Var k,a:Double;
    t:Longint;
Begin
     Inherited Create(ASampleRate);
     //Initialize global tables
     If not FreqTableInitialized Then
     Begin
         // build frequency table
         k:=1.059463094359; // 12th root of 2
         a:=6.875;
         a:=a*k;
         a:=a*k;
         a:=a*k; //Start with Note 0 (C)
         for t:=0 To FreqTableSize-1 Do
         Begin
              FreqTable[t]:=a;
              a:=a*k;
         End;
         FreqTableInitialized:=true;
     End;

     If not WaveFormsInitialized Then
     Begin
          // build waveforms tables
          For t:=0 To WaveTableSize-1 Do
          Begin
               SawToothTable[t]:=(-1.0+(2.0*(t/WaveTableSize)));
               If t<WaveTableSize Div 4 Then PulseTable[t]:=-1.0
               Else PulseTable[t]:=1.0; //1:3 pulse
          End;
          WaveFormsInitialized:=true;
     End;

     InitializeProgram;
End;

//Destructor - cleanup
Destructor TVSDemoSynth.Destroy;
Begin
     Inherited Destroy;
End;

//returns count of programs
Function TVSDemoSynth.GetProgramCount:LongInt;
Begin
     Result:=0; //we don't support programs, otherwise return the number of programs
End;

//returns program name
Procedure TVSDemoSynth.GetProgramName(Index:LongInt;Var Result:String);
Begin
     //returns a particular program name, Index ranges from 0..ProgramCount-1
     //since we don't support programs, we leave this function empty
End;

//sets program
Procedure TVSDemoSynth.SetProgram(NewValue:LongInt);
Begin
     //sets a particular program, Index ranges from 0..ProgramCount-1
     //since we don't support programs, we leave this function empty
End;

//returns group count
Function TVSDemoSynth.GetGroupCount:LongInt;
Begin
     //Synthesizer parameters are divided into groups.
     //Each group can have up to 16 parameters
     Result:=3; //we have 3 groups (General,Oscillator1 and Oscillator2)
End;

//returns group name
Procedure TVSDemoSynth.GetGroupName(Index:LongInt;Var Result:String);
Begin
    //returns a particular group name, Index ranges from 0..GroupCount-1
    Case Index Of
      0:Result:='General';
      1:Result:='Oscillator 1';
      2:Result:='Oscillator 2';
      Else Result:='';
    End;
End;

//returns parameter count
Function TVSDemoSynth.GetParamCount(GroupIndex:LongInt):LongInt;
Begin
     Case GroupIndex Of
       0:Result:=1;
       1,2:Result:=3;
       Else Result:=0;
     End;  
End;

//returns label for a parameter
Procedure TVSDemoSynth.GetParamLabel(GroupIndex,Index:LongInt;Var Result:String);
Begin
    //returns a label string that is displayed for a particular parameter
    //GroupIndex ranges from 0..GroupCount-1.
    //Index ranges from 0..ParamCount-1
    Case GroupIndex Of
      0: //General
      Begin
           Case Index Of
              0:Result:='Gain';
           End;
      End;
      1: //Oscillator1
      Begin
           Case Index Of
              0:Result:='Osc1 Wave';
              1:Result:='Osc1 Det';
              2:Result:='Osc1 Vol';
           End;
      End;
      2: //Oscillator 2
      Begin
           Case Index Of
              0:Result:='Osc2 Wave';
              1:Result:='Osc2 Det';
              2:Result:='Osc2 Vol';
           End;
      End;
    End; //case
End;

//returns display for a parameter
Procedure TVSDemoSynth.GetParamDisplay(GroupIndex,Index:LongInt;Var Result:String);
Begin
    //returns a display string that is displayed for the current value of a
    //particular parameter.
    //GroupIndex ranges from 0..GroupCount-1.
    //Index ranges from 0..ParamCount-1
    Case GroupIndex Of
      0: //General
      Begin
           Case Index Of
             0:Result:=Float2String(MasterGain*2);
           End;
      End;
      1: //Oscillator1
      Begin
           Case Index Of
             0:
             Begin
                  If (Osc1WaveForm<0.5) Then Result:='Saw'
                  Else Result:='Pulse';
             End;
             1:
             Begin
                  Result:=Int2String(Round(Osc1Detune*2400-1200)); //from -1200..1200 cents
                  Result:=Result+' cents';
             End;
             2:
             Begin
                  Result:=Int2String(Round(Osc1Volume*100));
                  Result:=Result+' %';
             End;
           End;  
      End;
      2: //Oscillator2
      Begin
           Case Index Of
             0:
             Begin
                  If (Osc2WaveForm<0.5) Then Result:='Saw'
                  Else Result:='Pulse';
             End;
             1:
             Begin
                  Result:=Int2String(Round(Osc2Detune*2400-1200)); //from -1200..1200 cents
                  Result:=Result+' cents';
             End;
             2:
             Begin
                  Result:=Int2String(Round(Osc2Volume*100));
                  Result:=Result+' %';
             End;
           End;
      End;
    End; //case
End;

//gets value of a parameter
Procedure TVSDemoSynth.GetParamValue(GroupIndex,Index:LongInt;Var Result:Float);
Begin
    //Parameter values range from -1..1
    //GroupIndex ranges from 0..GroupCount-1.
    //Index ranges from 0..ParamCount-1
    Case GroupIndex Of
      0: //General
      Begin
           Case Index Of
             0:Result:=MasterGain;
           End;
      End;
      1: //Oscillator1
      Begin
           Case Index Of
             0:Result:=Osc1WaveForm;
             1:Result:=Osc1Detune;
             2:Result:=Osc1Volume;
           End;
      End;
      2: //Oscillator 2
      Begin
           Case Index Of
             0:Result:=Osc2WaveForm;
             1:Result:=Osc2Detune;
             2:Result:=Osc2Volume;
           End;
      End;
    End; //case
End;

//sets value of a parameter
Procedure TVSDemoSynth.SetParamValue(GroupIndex,Index:LongInt;NewValue:Float);
Begin
    //Parameter values range from -1..1
    //GroupIndex ranges from 0..GroupCount-1.
    //Index ranges from 0..ParamCount-1
    Case GroupIndex Of
      0: //General
      Begin
           Case Index Of
             0:MasterGain:=NewValue;
           End;
      End;
      1: //Oscillator1
      Begin
           Case Index Of
             0:Osc1WaveForm:=NewValue;
             1:Osc1Detune:=NewValue;
             2:Osc1Volume:=NewValue;
           End;
      End;
      2: //Oscillator2
      Begin
           Case Index Of
             0:Osc2WaveForm:=NewValue;
             1:Osc2Detune:=NewValue;
             2:Osc2Volume:=NewValue;
           End;
      End;
    End; //case
End;

//returns size of Parameter record
Procedure TVSDemoSynth.GetParamsInfo(Var ParamsLen:LongInt);
Begin
    //returns the size of the buffer that is needed to store the
    //parameters of the synth. This value is used to calculate the
    //buffer size when calling "StoreParams"
    ParamsLen:=7*sizeof(float); //we have 7 parameters
End;

//stores parameters into buffer
Procedure TVSDemoSynth.StoreParams(Params:Pointer;ParamsLen:LongInt);
Begin
     If (ParamsLen<>7*sizeof(float)) Then exit; //Parameter size not accepted
     PFloatArray(Params)[0]:=Osc1WaveForm;
     PFloatArray(Params)[1]:=Osc1Detune;
     PFloatArray(Params)[2]:=Osc1Volume;
     PFloatArray(Params)[3]:=Osc2WaveForm;
     PFloatArray(Params)[4]:=Osc2Detune;
     PFloatArray(Params)[5]:=Osc2Volume;
     PFloatArray(Params)[6]:=MasterGain;
End;

//loads parameters from buffer
Procedure TVSDemoSynth.LoadParams(Params:Pointer;ParamsLen:LongInt);
Begin
    //The BufferSize passed matches the return value from "GetParamsInfo"
    if (ParamsLen<>7*sizeof(float)) Then exit; //Parameter size not accepted
    Osc1WaveForm:=PFloatArray(Params)[0];
    Osc1Detune:=PFloatArray(Params)[1];
    Osc1Volume:=PFloatArray(Params)[2];
    Osc2WaveForm:=PFloatArray(Params)[3];
    Osc2Detune:=PFloatArray(Params)[4];
    Osc2Volume:=PFloatArray(Params)[5];
    MasterGain:=PFloatArray(Params)[6];
End;

//Initializes current program with default values
Procedure TVSDemoSynth.InitializeProgram;
Begin
    //we don't support any programs, reset global parameters
    Osc1WaveForm:=0.0; //saw
    Osc1Detune:=0.5;
    Osc1Volume:=0.33;

    Osc2WaveForm:=1.0; //pulse
    Osc2Detune:=0.5;
    Osc2Volume:=0.33;

    MasterGain:=0.4;
End;

//returns the file extension used to load and store files for this synth.
//The string returned contains a three letter file extension, used to store
//and load parameter files (eg. "syn"). If the synth supports more than
//one file format, the extensions are separated by "|", e.g. "syn|dx7"
Procedure TVSDemoSynth.GetFileExt(Var Result:String);
Begin
     //we don't support load/save
     Result:='';
End;

//Save the synth parameters to the filename passed. If the function
//returns an error (-1), the error text can be copied to "FileName"
Function TVSDemoSynth.SaveProgram(Var FileName:String):LongInt;
Begin
     FileName:='This synth does not support loading/saving';
     Result:=-1; //we don't support load/save
End;

//Gets the Programs and Parameter lens for the given filename.
//Normally you can set "ProgramCount" to 1. If a given file contains more
//than one synthesizer program (as for the DX7 emulator of VSampler), you
//can assign a value>1 here.
//The "ParamsLen" argument must be altered with the parameter buffer len
//for each program. This information is used to allocate the buffers when
//calling "LoadProgram" (see below)
Procedure TVSDemoSynth.GetProgramInfo(Const FileName:String;Var ProgramCount,ParamsLen:LongInt);
Begin
    //we don't support loading/saving
    ProgramCount:=0;
    ParamsLen:=0;
End;

//Loads the synth parameters to the filename passed. If the function
//returns an error (-1), the error text can be copied to "FileName".
//Otherwise the "Result" parameter is an array of TSynthProgramInfo
//records. This array ranges from 0..ProgramCount (returned from "GetProgramInfo").
//Each record has a parameter pointer that was allocated by the caller with
//the "ParamsLen" return value from "GetProgramInfo". You should not reassign
//or free this pointer !
Function TVSDemoSynth.LoadProgram(Var FileName:String;Var Res:TSynthLoadProgramRec):LongInt;
Begin
     FileName:='This synth does not support loading/saving';
     Result:=-1; //we don't support load/save
End;

//This function is called each time a MIDI Note-On event arrives
//"Velocity" is the velocity of the note played and can be used for realtime
//control of synth parameters
//"ScaleCents" is the scaling of the note in cents that should be added
//to the note frequency (100 cents=1 semitone, 1200 cents=1 octave)
//The values passed must be stored for later calls of the "GetBuffer"
//method when Virtual Sampler actually passes data to the soundcard
Procedure TVSDemoSynth.NoteOn(Note,Velocity,ScaleCents:LongInt);
Var CentsOsc1,CentsOsc2,NoteOsc1,NoteOsc2:LongInt;
    Freq1Osc1,Freq1Osc2,Scale:Double;
Begin
     MidiNote:=Note;

     MidiVelocity:=Velocity;
     MidiScaleCents:=ScaleCents;

     //calculate delta cents
     CentsOsc1:=ScaleCents+Round(Osc1Detune*2400-1200);
     CentsOsc2:=ScaleCents+Round(Osc2Detune*2400-1200);

     NoteOsc1:=Note+CentsOsc1 Div 100;
     NoteOsc2:=Note+CentsOsc2 Div 100;

     CentsOsc1:=CentsOsc1 Mod 100;
     CentsOsc2:=CentsOsc2 Mod 100;

     //Calculate base frequencies for each oscillator
     FreqOsc1:=FreqTable[NoteOsc1 And 127];
     If CentsOsc1<0 Then Freq1Osc1:=FreqTable[(NoteOsc1-1) And 127]
     Else Freq1Osc1:=FreqTable[(NoteOsc1+1) And 127];
     FreqOsc1:=FreqOsc1+((Freq1Osc1-FreqOsc1)*CentsOsc1)/100;

     FreqOsc2:=FreqTable[NoteOsc2 And 127];
     If CentsOsc2<0 Then Freq1Osc2:=FreqTable[(NoteOsc2-1) And 127]
     Else Freq1Osc2:=FreqTable[(NoteOsc2+1) And 127];
     FreqOsc2:=FreqOsc2+((Freq1Osc2-FreqOsc2)*CentsOsc2)/100;

     //scale the frequencies according to the current SampleRate
     Scale:=WaveTableSize/SampleRate;
     FreqOsc1:=FreqOsc1*Scale;
     FreqOsc2:=FreqOsc2*Scale;

     //calculate volume depending on current Velocity
     Volume := MasterGain * Velocity*(1.0/127.0);

     //setup processing
     PhaseOsc1:=0.0;
     PhaseOsc2:=0.0;
     MidiNoteIsOn:=true;
End;

//This function is called each time a MIDI Note-Off event arrives
Procedure TVSDemoSynth.NoteOff;
Begin
     MidiNoteIsOn:=false;
End;

//This is probably the most important function since it does the actual
//sample processing work. The buffer is an array of float values 0..SampleCount-1.
//The sample range is -1..1. This function returns the sample amount copied
//to the buffer. This function replaces the samples in the buffer passed by the caller

//Note: This function is called each time Virtual Sampler sends a buffer to
//      the soundcard and therefore should be optimized for speed
Function TVSDemoSynth.GetBuffer(Buffer:PFloatArray;SampleCount:LongInt):LongInt;
Var Frames:LongInt;
    Wave1,Wave2:PFloatArray;
Begin
     if not MidiNoteIsOn Then
     Begin
          FillChar(Buffer^,SampleCount*sizeof(float),0);
          Result:=0; //we have done
          exit;
     End;

     Frames:=SampleCount;

     If Osc1WaveForm<0.5 Then Wave1:=@SawToothTable
     Else Wave1:=@PulseTable;
     If Osc2WaveForm<0.5 Then Wave2:=@SawToothTable
     Else Wave2:=@PulseTable;

     While Frames>0 Do
     Begin
          Buffer^[0]:=(wave1[Round(PhaseOsc1) And (WaveTableSize-1)]*Osc1Volume*Volume);
          Buffer^[0]:=Buffer^[0]+(wave2[Round(PhaseOsc2) And (WaveTableSize-1)]*Osc2Volume*Volume);
          inc(Cardinal(Buffer),sizeof(Float));

          PhaseOsc1:=PhaseOsc1+FreqOsc1;
          PhaseOsc2:=PhaseOsc2+FreqOsc2;

          dec(Frames);
    End;

    Result:=SampleCount; //we have processed all samples
End;

//*******************************************************************
//*																	*
//* Exported Functions												*
//*																	*
//*																	*	
//*******************************************************************


//Note: The following three functions must always be exported by
//      a Virtual Sampler Synth PlugIn. Otherwise the Synth will not
//      be recognized by Virtual Sampler, These three functions must
//      also be part of your "Exports" section !

Procedure VSSynthGetInfo(Var Params:TPlugInSynth);stdcall;
Begin
 //you always must copy the default parameters (defined in
 //vssynthbase.h to the record passes by the caller
 //You only should modify the name of the default record and
 //create a class derived from VSSynthBase for your Synth.
 Params:=SynthParams;

 //modify the name of the Synth
 Params.Name:='VS PlugIn Demo Synth';
End;

//create a synthesizer Instance
Function VSSynthCreateInstance(SampleRate:LongInt):TVSSynthBase;stdcall;
Begin
     Result:=TVSDemoSynth.Create(SampleRate);
End;

//destroys an instance
Procedure VSSynthDestroyInstance(Instance:TVSSynthBase);stdcall;
Begin
     Instance.Free;
End;

Exports
   VSSynthGetInfo,
   VSSynthCreateInstance,
   VSSynthDestroyInstance;

begin
end.
